module Comune
  class GenerateToken
    END_POINT = 'https://accounts.google.com/o/oauth2/token'
    GRANT_TYPE = 'refresh_token'
    MAXIMUM_READ_TIMEOUT = 30
    MAXIMUM_OPEN_TIMEOUT = 30

    class << self
      def execute
        request_to_google['access_token']
      end

      private

      def request_to_google
        uri = URI.parse(END_POINT)
        req = Net::HTTP::Post.new(uri)
        req.content_type = 'application/json'
        req.body = JSON.dump(grant_type:    GRANT_TYPE,
                             client_id:     client_id,
                             client_secret: client_secret,
                             refresh_token: refresh_token)
        req_options = { use_ssl: uri.scheme == 'https' }
        res = Net::HTTP.start(uri.host, uri.port, req_options) do |http|
          http.read_timeout = MAXIMUM_READ_TIMEOUT
          http.open_timeout = MAXIMUM_OPEN_TIMEOUT
          http.request(req)
        end
        JSON.parse(res.body)
      end

      def client_id
        Comune.client_id
      end

      def client_secret
        Comune.client_secret
      end

      def refresh_token
        Comune.refresh_token
      end
    end
  end
end
