require 'json'

module Comune
  class Receipt
    MAXIMUM_READ_TIMEOUT = 30
    MAXIMUM_OPEN_TIMEOUT = 30

    def initialize(receipt_data, signature)
      @receipt_data = receipt_data
      @signature    = signature
    end

    def verify
      Comune::Tampering.verify(@receipt_data, @signature)
      access_token = Comune::GenerateToken.execute
      receipt_verification(access_token)
    end

    private

    def purchase_token
      encoded_receipt_data['purchaseToken']
    end

    def encoded_receipt_data
      JSON.parse(Base64.decode64(@receipt_data))
    end

    def receipt_verification(access_token)
      uri = URI.parse("#{end_point}/tokens/#{purchase_token}")
      req = Net::HTTP::Get.new(uri.path)
      req.content_type = 'application/json'
      req['Authorization'] = "Bearer #{access_token}"
      req_options = { use_ssl: uri.scheme == 'https' }
      res = Net::HTTP.start(uri.host, uri.port, req_options) do |http|
        http.read_timeout = MAXIMUM_READ_TIMEOUT
        http.open_timeout = MAXIMUM_OPEN_TIMEOUT
        http.request(req)
      end
      JSON.parse(res.body)
    end

    def end_point
      "https://www.googleapis.com/androidpublisher/v3/applications/#{Comune.package_name}/purchases/subscriptions/#{Comune.subscription_id}"
    end
  end
end
